@@ -376,6 +376,11 @@ int renderSVGSymbolCairo(imageObj *img, double x, double y, symbolObj *symbol,
   return MS_SUCCESS;
 
 #else
+  (void)img;
+  (void)x;
+  (void)y;
+  (void)symbol;
+  (void)style;
   msSetError(MS_MISCERR,
              "SVG Symbols requested but is not built with libsvgcairo",
              "renderSVGSymbolCairo()");
@@ -536,7 +541,7 @@ imageObj *createImageCairo(int width, int height, outputFormatObj *format,
  * geospatial */
 /* information to the regular PDF generated by cairo. This is only triggered if
  * the */
-/* GEO_ENCODING outputformat option is set (to ISO32000 or OGC_BP). Additionnal
+/* GEO_ENCODING outputformat option is set (to ISO32000 or OGC_BP). Additional
  */
 /* options can be provided by specifying outputformat options starting with */
 /* METADATA_ITEM: prefix. For example METADATA_ITEM:PRODUCER=MapServer */
@@ -840,6 +845,14 @@ static cairo_operator_t ms2cairo_compop(CompositingOperation op) {
     return CAIRO_OPERATOR_DIFFERENCE;
   case MS_COMPOP_EXCLUSION:
     return CAIRO_OPERATOR_EXCLUSION;
+  case MS_COMPOP_HSL_HUE:
+    return CAIRO_OPERATOR_HSL_HUE;
+  case MS_COMPOP_HSL_LUMINOSITY:
+    return CAIRO_OPERATOR_HSL_LUMINOSITY;
+  case MS_COMPOP_HSL_SATURATION:
+    return CAIRO_OPERATOR_HSL_SATURATION;
+  case MS_COMPOP_HSL_COLOR:
+    return CAIRO_OPERATOR_HSL_COLOR;
   case MS_COMPOP_INVERT:
   case MS_COMPOP_INVERT_RGB:
   case MS_COMPOP_MINUS:
@@ -921,6 +934,8 @@ void freeSVGCache(symbolObj *s) {
   }
   msFree(s->renderer_cache);
   s->renderer_cache = NULL;
+#else
+  (void)s;
 #endif
 }
 
@@ -1048,6 +1063,7 @@ int msPreloadSVGSymbol(symbolObj *symbol) {
   return MS_SUCCESS;
 
 #else
+  (void)symbol;
   msSetError(MS_MISCERR,
              "SVG Symbols requested but is not built with libsvgcairo",
              "msPreloadSVGSymbol()");
@@ -1082,7 +1098,7 @@ int msRenderRasterizedSVGSymbol(imageObj *img, double x, double y,
       svg_cache->pixmap_buffer = msSmallCalloc(1, sizeof(rasterBufferObj));
     }
 
-    // increase pixmap size to accomodate scaling/rotation
+    // increase pixmap size to accommodate scaling/rotation
     if (style->scale != 1.0) {
       width = surface_w = (symbol->sizex * style->scale + 0.5);
       height = surface_h = (symbol->sizey * style->scale + 0.5);
@@ -1143,6 +1159,11 @@ int msRenderRasterizedSVGSymbol(imageObj *img, double x, double y,
   MS_IMAGE_RENDERER(img)->freeSymbol(&pixsymbol);
   return status;
 #else
+  (void)img;
+  (void)x;
+  (void)y;
+  (void)symbol;
+  (void)style;
   msSetError(
       MS_MISCERR,
       "SVG Symbols requested but MapServer is not built with libsvgcairo",
@@ -1185,6 +1206,7 @@ int msPopulateRendererVTableCairoRaster(rendererVTableObj *renderer) {
   renderer->cleanup = &cleanupCairo;
   return MS_SUCCESS;
 #else
+  (void)renderer;
   msSetError(MS_MISCERR, "Cairo Driver requested but MapServer is not built in",
              "msPopulateRendererVTableCairoRaster()");
   return MS_FAILURE;
@@ -1223,6 +1245,7 @@ int populateRendererVTableCairoVector(rendererVTableObj *renderer) {
   renderer->cleanup = &cleanupCairo;
   return MS_SUCCESS;
 #else
+  (void)renderer;
   msSetError(MS_MISCERR, "Cairo Driver requested but MapServer is not built in",
              "msPopulateRendererVTableCairoRaster()");
   return MS_FAILURE;
