@@ -323,11 +323,9 @@ int pieLayerProcessDynamicDiameter(layerObj *layer) {
   char *space = strchr(attrib, ' ');
   if (space) {
     *space = '\0';
-    switch (sscanf(space + 1, "%lf %lf %lf %lf", &mindiameter, &maxdiameter,
-                   &minvalue, &maxvalue)) {
-    case 4: /*we have the attribute and the four range values*/
-      break;
-    default:
+    if (sscanf(space + 1, "%lf %lf %lf %lf", &mindiameter, &maxdiameter,
+               &minvalue, &maxvalue) != 4) {
+      /*we don't have the attribute and the four range values*/
       free(attrib);
       msSetError(MS_MISCERR,
                  "Chart Layer format error for processing key \"CHART_RANGE\"",
@@ -396,8 +394,17 @@ int msDrawPieChartLayer(mapObj *map, layerObj *layer, imageObj *image) {
     if (chartRangeProcessingKey == NULL)
       diameter = 20;
     else {
-      sscanf(chartRangeProcessingKey, "%*s %lf %lf %lf %lf %lf", &mindiameter,
-             &maxdiameter, &minvalue, &maxvalue, &exponent);
+      const int nvalues =
+          sscanf(chartRangeProcessingKey, "%*s %lf %lf %lf %lf %lf",
+                 &mindiameter, &maxdiameter, &minvalue, &maxvalue, &exponent);
+      if (nvalues != 4 && nvalues != 5) {
+        msSetError(
+            MS_MISCERR,
+            "msDrawChart format error for processing key \"CHART_SIZE_RANGE\": "
+            "itemname minsize maxsize minval maxval [exponent] is expected",
+            "msDrawPieChartLayer()");
+        return MS_FAILURE;
+      }
     }
   } else {
     if (sscanf(chartSizeProcessingKey, "%lf", &diameter) != 1) {
@@ -554,13 +561,10 @@ int msDrawBarChartLayer(mapObj *map, layerObj *layer, imageObj *image) {
   if (chartSizeProcessingKey == NULL) {
     width = height = 20;
   } else {
-    switch (sscanf(chartSizeProcessingKey, "%lf %lf", &width, &height)) {
-    case 2:
-      break;
-    case 1:
+    const int ret = sscanf(chartSizeProcessingKey, "%lf %lf", &width, &height);
+    if (ret == 1) {
       height = width;
-      break;
-    default:
+    } else if (ret != 2) {
       msSetError(MS_MISCERR,
                  "msDrawChart format error for processing key \"CHART_SIZE\"",
                  "msDrawBarChartLayer()");
