@@ -514,14 +514,14 @@ static size_t msOGRStdoutWriteFunction(const void *ptr, size_t size,
 }
 
 /************************************************************************/
-/*                      msOGROutputGetAdditonalFiles()                  */
+/*                      msOGROutputGetAdditionalFiles()                  */
 /*                                                                      */
 /*  Collect additional files specified in                               */
 /*  wfs/ows_additional_files_in_output of WEB.METADATA and LAYER.METADATA */
 /************************************************************************/
 
 /* Result to be freed with CSLDestroy() */
-static char **msOGROutputGetAdditonalFiles(mapObj *map) {
+static char **msOGROutputGetAdditionalFiles(mapObj *map) {
   int i;
   hashTableObj *hSetAdditionalFiles;
   char **papszFiles = NULL;
@@ -1209,7 +1209,7 @@ int msOGRWriteFromQuery(mapObj *map, outputFormatObj *format, int sendheaders)
     msIO_sendHeaders();
     msIO_fprintf(stdout, "--%s\r\n", boundary);
 
-    papszAdditionalFiles = msOGROutputGetAdditonalFiles(map);
+    papszAdditionalFiles = msOGROutputGetAdditionalFiles(map);
     file_list = msCSLConcatenate(file_list, papszAdditionalFiles);
     CSLDestroy(papszAdditionalFiles);
 
@@ -1257,7 +1257,7 @@ int msOGRWriteFromQuery(mapObj *map, outputFormatObj *format, int sendheaders)
 
     hZip = CPLCreateZip(zip_filename, NULL);
 
-    papszAdditionalFiles = msOGROutputGetAdditonalFiles(map);
+    papszAdditionalFiles = msOGROutputGetAdditionalFiles(map);
     file_list = msCSLConcatenate(file_list, papszAdditionalFiles);
     CSLDestroy(papszAdditionalFiles);
 
