@@ -76,6 +76,7 @@ struct projectionContext {
   int ref_count;
   pjCacheEntry pj_cache[PJ_CACHE_ENTRY_SIZE];
   int pj_cache_size;
+  int cannotFindProjDb;
 };
 
 /************************************************************************/
@@ -327,7 +328,9 @@ static PJ *getBaseGeographicCRS(projectionObj *in) {
 /************************************************************************/
 
 static void msProjErrorLogger(void *user_data, int level, const char *message) {
-  (void)user_data;
+  projectionContext *ctx = (projectionContext *)user_data;
+  if (strstr(message, "Cannot find proj.db"))
+    ctx->cannotFindProjDb = MS_TRUE;
   if (level == PJ_LOG_ERROR && msGetGlobalDebugLevel() >= MS_DEBUGLEVEL_VV) {
     msDebug("PROJ: Error: %s\n", message);
   } else if (level == PJ_LOG_DEBUG) {
@@ -335,6 +338,25 @@ static void msProjErrorLogger(void *user_data, int level, const char *message) {
   }
 }
 
+/************************************************************************/
+/*                        msProjGetProjErrorString()                    */
+/************************************************************************/
+
+static const char *msProjGetProjErrorString(projectionObj *p) {
+  const int l_pj_errno = proj_context_errno(p->proj_ctx->proj_ctx);
+#if PROJ_VERSION_MAJOR >= 8
+  const char *projErrorStr =
+      proj_context_errno_string(p->proj_ctx->proj_ctx, l_pj_errno);
+#else
+  const char *projErrorStr = proj_errno_string(l_pj_errno);
+#endif
+  if (p->proj_ctx->cannotFindProjDb) {
+    projErrorStr = "Cannot find proj.db";
+    p->proj_ctx->cannotFindProjDb = MS_FALSE;
+  }
+  return projErrorStr;
+}
+
 /************************************************************************/
 /*                        msProjectionContextCreate()                   */
 /************************************************************************/
@@ -350,7 +372,7 @@ projectionContext *msProjectionContextCreate(void) {
   }
   ctx->ref_count = 1;
   proj_context_use_proj4_init_rules(ctx->proj_ctx, TRUE);
-  proj_log_func(ctx->proj_ctx, NULL, msProjErrorLogger);
+  proj_log_func(ctx->proj_ctx, ctx, msProjErrorLogger);
   return ctx;
 }
 
@@ -432,7 +454,7 @@ reprojectionObj *msProjectCreateReprojector(projectionObj *in,
     PJ *pj = NULL;
 
     if ((in == NULL || in->proj == NULL) && out &&
-        out->proj) { /* input coordinates are lat/lon */
+        out->proj) { /* input coordinates are lat/long */
       PJ *source_crs = getBaseGeographicCRS(out);
       projectionObj in_modified;
       memset(&in_modified, 0, sizeof(in_modified));
@@ -490,6 +512,7 @@ int msProjectTransformPoints(reprojectionObj *reprojector, int npoints,
 */
 int msInitProjection(projectionObj *p) {
   p->gt.need_geotransform = MS_FALSE;
+  p->is_polar = -1;
   p->numargs = 0;
   p->args = NULL;
   p->wellknownprojection = wkp_none;
@@ -508,6 +531,7 @@ void msFreeProjection(projectionObj *p) {
   p->proj_ctx = NULL;
 
   p->gt.need_geotransform = MS_FALSE;
+  p->is_polar = -1;
   p->wellknownprojection = wkp_none;
   msFreeCharArray(p->args, p->numargs);
   p->args = NULL;
@@ -676,10 +700,8 @@ static int _msProcessAutoProjection(projectionObj *p) {
   args = msStringSplit(szProjBuf, '+', &numargs);
 
   if (!(p->proj = proj_create_argv(p->proj_ctx->proj_ctx, numargs, args))) {
-    int l_pj_errno = proj_context_errno(p->proj_ctx->proj_ctx);
-    msSetError(MS_PROJERR, "proj error \"%s\" for \"%s\"",
-               "msProcessProjection()", proj_errno_string(l_pj_errno),
-               szProjBuf);
+    msSetError(MS_PROJERR, "PROJ error \"%s\" when instantiating \"%s\"",
+               "msProcessProjection()", msProjGetProjErrorString(p), szProjBuf);
     msFreeCharArray(args, numargs);
     return (-1);
   }
@@ -748,9 +770,8 @@ int msProcessProjection(projectionObj *p) {
     /* Deal e.g. with EPSG:XXXX or ESRI:XXXX */
     if (!(p->proj =
               proj_create_argv(p->proj_ctx->proj_ctx, p->numargs, p->args))) {
-      int l_pj_errno = proj_context_errno(p->proj_ctx->proj_ctx);
-      msSetError(MS_PROJERR, "proj error \"%s\" for \"%s\"",
-                 "msProcessProjection()", proj_errno_string(l_pj_errno),
+      msSetError(MS_PROJERR, "PROJ error \"%s\" when instantiating \"%s\"",
+                 "msProcessProjection()", msProjGetProjErrorString(p),
                  p->args[0]);
       return (-1);
     }
@@ -790,14 +811,13 @@ int msProcessProjection(projectionObj *p) {
       fprintf(stderr, "\n");
 #endif
     if (!(p->proj = proj_create_argv(p->proj_ctx->proj_ctx, numargs, args))) {
-      int l_pj_errno = proj_context_errno(p->proj_ctx->proj_ctx);
       if (p->numargs > 1) {
-        msSetError(MS_PROJERR, "proj error \"%s\" for \"%s:%s\"",
-                   "msProcessProjection()", proj_errno_string(l_pj_errno),
+        msSetError(MS_PROJERR, "PROJ error \"%s\" when instantiating \"%s:%s\"",
+                   "msProcessProjection()", msProjGetProjErrorString(p),
                    p->args[0], p->args[1]);
       } else {
-        msSetError(MS_PROJERR, "proj error \"%s\" for \"%s\"",
-                   "msProcessProjection()", proj_errno_string(l_pj_errno),
+        msSetError(MS_PROJERR, "PROJ error \"%s\" when instantiating \"%s\"",
+                   "msProcessProjection()", msProjGetProjErrorString(p),
                    p->args[0]);
       }
       free(args);
@@ -1703,7 +1723,7 @@ int msProjectRectAsPolygon(reprojectionObj *reprojector, rectObj *rect) {
   /* If there is more than two sample points we will also get samples from the
    * diagonal line */
   ringPoints =
-      (pointObj *)calloc(sizeof(pointObj), NUMBER_OF_SAMPLE_POINTS * 5 + 3);
+      (pointObj *)calloc(NUMBER_OF_SAMPLE_POINTS * 5 + 3, sizeof(pointObj));
   ring.point = ringPoints;
   ring.numpoints = 0;
 
@@ -1854,15 +1874,18 @@ int msProjectRect(projectionObj *in, projectionObj *out, rectObj *rect) {
   reprojectionObj *reprojector = NULL;
 
   /* Detect projecting from polar stereographic to longlat */
-  if (in && !in->gt.need_geotransform && out && !out->gt.need_geotransform &&
-      !msProjIsGeographicCRS(in) && msProjIsGeographicCRS(out)) {
+  if (in && in->is_polar < 0 && !in->gt.need_geotransform && out &&
+      !out->gt.need_geotransform && !msProjIsGeographicCRS(in) &&
+      msProjIsGeographicCRS(out)) {
     reprojector = msProjectCreateReprojector(in, out);
+    in->is_polar = 0;
     for (int sign = 1; sign >= -1; sign -= 2) {
       pointObj p;
       p.x = 0.0;
       p.y = 0.0;
       if (reprojector && msProjectPointEx(reprojector, &p) == MS_SUCCESS &&
           fabs(p.y - sign * 90) < 1e-8) {
+        in->is_polar = 1;
         /* Is the pole in the rectangle ? */
         if (0 >= rect->minx && 0 >= rect->miny && 0 <= rect->maxx &&
             0 <= rect->maxy) {
@@ -1885,11 +1908,12 @@ int msProjectRect(projectionObj *in, projectionObj *out, rectObj *rect) {
   }
 
   /* Detect projecting from polar stereographic to another projected system */
-  else if (in && !in->gt.need_geotransform && out &&
+  else if (in && in->is_polar < 0 && !in->gt.need_geotransform && out &&
            !out->gt.need_geotransform && !msProjIsGeographicCRS(in) &&
            !msProjIsGeographicCRS(out)) {
     reprojectionObj *reprojectorToLongLat =
         msProjectCreateReprojector(in, NULL);
+    in->is_polar = 0;
     for (int sign = 1; sign >= -1; sign -= 2) {
       pointObj p;
       p.x = 0.0;
@@ -1897,6 +1921,7 @@ int msProjectRect(projectionObj *in, projectionObj *out, rectObj *rect) {
       if (reprojectorToLongLat &&
           msProjectPointEx(reprojectorToLongLat, &p) == MS_SUCCESS &&
           fabs(p.y - 90) < 1e-8) {
+        in->is_polar = 1;
         reprojector = msProjectCreateReprojector(in, out);
         /* Is the pole in the rectangle ? */
         if (0 >= rect->minx && 0 >= rect->miny && 0 <= rect->maxx &&
@@ -2416,7 +2441,7 @@ static int ConvertProjUnitStringToMS(const char *pszProjUnit) {
 /*           int GetMapserverUnitUsingProj(projectionObj *psProj)       */
 /*                                                                      */
 /*      Return a mapserver unit corresponding to the projection         */
-/*      passed. Retunr -1 on failure                                    */
+/*      passed. Return -1 on failure                                    */
 /************************************************************************/
 int GetMapserverUnitUsingProj(projectionObj *psProj) {
   if (msProjIsGeographicCRS(psProj))
@@ -2495,6 +2520,7 @@ projectionContext *msProjectionContextGetFromPool() {
   if (headOfLinkedListOfProjContext) {
     LinkedListOfProjContext *next = headOfLinkedListOfProjContext->next;
     context = headOfLinkedListOfProjContext->context;
+    context->thread_id = msGetThreadId();
     msFree(headOfLinkedListOfProjContext);
     headOfLinkedListOfProjContext = next;
   } else {
