@@ -82,6 +82,8 @@ void msCGIWriteError(mapservObj *mapserv) {
     if (CPLGetConfigOption("MS_ERROR_URL", NULL) != NULL) {
       msRedirect(CPLGetConfigOption("MS_ERROR_URL", NULL));
     } else {
+      if (ms_error->http_status[0])
+        msIO_setHeader("Status", "%s", ms_error->http_status);
       msIO_setHeader("Content-Type", "text/html");
       msIO_sendHeaders();
       msIO_printf("<HTML>\n");
@@ -107,6 +109,8 @@ void msCGIWriteError(mapservObj *mapserv) {
       url = CPLGetConfigOption("MS_ERROR_URL", NULL);
     msRedirect(url);
   } else {
+    if (ms_error->http_status[0])
+      msIO_setHeader("Status", "%s", ms_error->http_status);
     msIO_setHeader("Content-Type", "text/html");
     msIO_sendHeaders();
     msIO_printf("<HTML>\n");
@@ -167,7 +171,7 @@ mapObj *msCGILoadMap(mapservObj *mapserv, configObj *config) {
   int i;
   mapObj *map = NULL;
 
-  const char *ms_map_bad_pattern_default = "[/\\]{2}|[/\\]?\\.+[/\\]|,";
+  const char *ms_map_bad_pattern_default = "[/\\\\]{2}|[/\\\\]?\\.+[/\\\\]|,";
 
   int ms_mapfile_tainted = MS_TRUE;
   const char *ms_mapfile = CPLGetConfigOption("MS_MAPFILE", NULL);
@@ -182,6 +186,20 @@ mapObj *msCGILoadMap(mapservObj *mapserv, configObj *config) {
 
   const char *map_value = NULL;
 
+  // Determine if it is a WMS query so that errors emitted look for the
+  // MS_WMS_ERROR_STATUS_CODE configuration option to determine if a HTTP
+  // status must be emitted. Otherwise MS_HTTP_ERROR_STATUS_CODE is used.
+  {
+    int isWMS = MS_FALSE;
+    for (i = 0; i < mapserv->request->NumParams; i++) {
+      if (strcasecmp(mapserv->request->ParamNames[i], "SERVICE") == 0) {
+        isWMS = strcasecmp(mapserv->request->ParamValues[i], "WMS") == 0;
+        break;
+      }
+    }
+    msSetErrorSetIsWMS(isWMS);
+  }
+
   if (mapserv->request->api_path != NULL) {
     map_value = mapserv->request
                     ->api_path[0]; /* mapfile is *always* in the first position
@@ -198,8 +216,9 @@ mapObj *msCGILoadMap(mapservObj *mapserv, configObj *config) {
 
   if (map_value == NULL) {
     if (ms_mapfile == NULL) {
-      msSetError(MS_WEBERR, "CGI variable \"map\" is not set.",
-                 "msCGILoadMap()"); /* no default, outta here */
+      msSetErrorWithStatus(MS_WEBERR, MS_HTTP_500_INTERNAL_SERVER_ERROR,
+                           "CGI variable \"map\" is not set.",
+                           "msCGILoadMap()"); /* no default, outta here */
       return NULL;
     }
     ms_mapfile_tainted = MS_FALSE;
@@ -213,10 +232,11 @@ mapObj *msCGILoadMap(mapservObj *mapserv, configObj *config) {
       /* by now we know the map parameter isn't referencing something in the
        * configuration */
       if (ms_map_no_path != NULL) {
-        msSetError(MS_WEBERR,
-                   "CGI variable \"map\" not found in configuration and this "
-                   "server is not configured for full paths.",
-                   "msCGILoadMap()");
+        msSetErrorWithStatus(
+            MS_WEBERR, MS_HTTP_500_INTERNAL_SERVER_ERROR,
+            "CGI variable \"map\" not found in configuration and this "
+            "server is not configured for full paths.",
+            "msCGILoadMap()");
         return NULL;
       }
       ms_mapfile = map_value;
@@ -226,20 +246,23 @@ mapObj *msCGILoadMap(mapservObj *mapserv, configObj *config) {
   /* validate ms_mapfile if tainted */
   if (ms_mapfile_tainted == MS_TRUE) {
     if (ms_map_pattern == NULL) { // can't go any further, bail
-      msSetError(MS_WEBERR,
-                 "Required configuration value MS_MAP_PATTERN not set.",
-                 "msCGILoadMap()");
+      msSetErrorWithStatus(
+          MS_WEBERR, MS_HTTP_500_INTERNAL_SERVER_ERROR,
+          "Required configuration value MS_MAP_PATTERN not set.",
+          "msCGILoadMap()");
       return NULL;
     }
     if (msIsValidRegex(ms_map_bad_pattern) == MS_FALSE ||
         msEvalRegex(ms_map_bad_pattern, ms_mapfile) == MS_TRUE) {
-      msSetError(MS_WEBERR, "CGI variable \"map\" fails to validate.",
-                 "msCGILoadMap()");
+      msSetErrorWithStatus(MS_WEBERR, MS_HTTP_500_INTERNAL_SERVER_ERROR,
+                           "CGI variable \"map\" fails to validate.",
+                           "msCGILoadMap()");
       return NULL;
     }
     if (msEvalRegex(ms_map_pattern, ms_mapfile) != MS_TRUE) {
-      msSetError(MS_WEBERR, "CGI variable \"map\" fails to validate.",
-                 "msCGILoadMap()");
+      msSetErrorWithStatus(MS_WEBERR, MS_HTTP_500_INTERNAL_SERVER_ERROR,
+                           "CGI variable \"map\" fails to validate.",
+                           "msCGILoadMap()");
       return NULL;
     }
   }
@@ -285,8 +308,8 @@ mapObj *msCGILoadMap(mapservObj *mapserv, configObj *config) {
               ms_context_bad_pattern = ms_map_bad_pattern_default;
 
             if (ms_context_pattern == NULL) { // can't go any further, bail
-              msSetError(
-                  MS_WEBERR,
+              msSetErrorWithStatus(
+                  MS_WEBERR, MS_HTTP_500_INTERNAL_SERVER_ERROR,
                   "Required configuration value MS_CONTEXT_PATTERN not set.",
                   "msCGILoadMap()");
               msFreeMap(map);
@@ -295,17 +318,19 @@ mapObj *msCGILoadMap(mapservObj *mapserv, configObj *config) {
             if (msIsValidRegex(ms_context_bad_pattern) == MS_FALSE ||
                 msEvalRegex(ms_context_bad_pattern, map_context_filename) ==
                     MS_TRUE) {
-              msSetError(MS_WEBERR,
-                         "CGI variable \"context\" fails to validate.",
-                         "msCGILoadMap()");
+              msSetErrorWithStatus(
+                  MS_WEBERR, MS_HTTP_500_INTERNAL_SERVER_ERROR,
+                  "CGI variable \"context\" fails to validate.",
+                  "msCGILoadMap()");
               msFreeMap(map);
               return NULL;
             }
             if (msEvalRegex(ms_context_pattern, map_context_filename) !=
                 MS_TRUE) {
-              msSetError(MS_WEBERR,
-                         "CGI variable \"context\" fails to validate.",
-                         "msCGILoadMap()");
+              msSetErrorWithStatus(
+                  MS_WEBERR, MS_HTTP_500_INTERNAL_SERVER_ERROR,
+                  "CGI variable \"context\" fails to validate.",
+                  "msCGILoadMap()");
               msFreeMap(map);
               return NULL;
             }
@@ -382,7 +407,7 @@ int msCGISetMode(mapservObj *mapserv) {
 }
 
 /*
-** API-related funtions.
+** API-related functions.
 */
 int msCGIIsAPIRequest(mapservObj *mapserv) {
   char **tmp_api_path = NULL;
@@ -583,7 +608,6 @@ static int commonLoadForm(mapservObj *mapserv, mapObj *map) {
 int msCGILoadForm(mapservObj *mapserv) {
   int i, n;
   char **tokens = NULL;
-  int rosa_type = 0;
   double tmpval;
   char *strtoderr;
 
@@ -1361,61 +1385,6 @@ int msCGILoadForm(mapservObj *mapserv) {
       continue;
     }
 
-    /* -------------------------------------------------------------------- */
-    /*      The following code is used to support the rosa applet (for      */
-    /*      more information on Rosa, please consult :                      */
-    /*      http://www.maptools.org/rosa/) .                                */
-    /*      This code was provided by Tim.Mackey@agso.gov.au.               */
-    /*                                                                      */
-    /*      For Application using it can be seen at :                       */
-    /*        http://www.agso.gov.au/map/pilbara/                           */
-    /*                                                                      */
-    /* -------------------------------------------------------------------- */
-
-    if (strcasecmp(mapserv->request->ParamNames[i], "INPUT_TYPE") == 0) {
-      /* Rosa input type */
-      if (strcasecmp(mapserv->request->ParamValues[i], "auto_rect") == 0) {
-        rosa_type = 1; /* rectangle */
-        continue;
-      }
-
-      if (strcasecmp(mapserv->request->ParamValues[i], "auto_point") == 0) {
-        rosa_type = 2; /* point */
-        continue;
-      }
-    }
-    if (strcasecmp(mapserv->request->ParamNames[i], "INPUT_COORD") == 0) {
-      /* Rosa coordinates */
-
-      switch (rosa_type) {
-      case 1:
-        sscanf(mapserv->request->ParamValues[i], "%lf,%lf;%lf,%lf",
-               &mapserv->ImgBox.minx, &mapserv->ImgBox.miny,
-               &mapserv->ImgBox.maxx, &mapserv->ImgBox.maxy);
-        if ((mapserv->ImgBox.minx != mapserv->ImgBox.maxx) &&
-            (mapserv->ImgBox.miny != mapserv->ImgBox.maxy)) {
-          mapserv->CoordSource = FROMIMGBOX;
-          mapserv->QueryCoordSource = FROMIMGBOX;
-        } else {
-          mapserv->CoordSource = FROMIMGPNT;
-          mapserv->QueryCoordSource = FROMIMGPNT;
-          mapserv->ImgPnt.x = mapserv->ImgBox.minx;
-          mapserv->ImgPnt.y = mapserv->ImgBox.miny;
-        }
-        break;
-      case 2:
-        sscanf(mapserv->request->ParamValues[i], "%lf,%lf", &mapserv->ImgPnt.x,
-               &mapserv->ImgPnt.y);
-        mapserv->CoordSource = FROMIMGPNT;
-        mapserv->QueryCoordSource = FROMIMGPNT;
-        break;
-      }
-      continue;
-    }
-    /* -------------------------------------------------------------------- */
-    /*      end of code for Rosa support.                                   */
-    /* -------------------------------------------------------------------- */
-
   } /* next parameter */
 
   if (mapserv->Mode == ZOOMIN) {
@@ -1567,7 +1536,7 @@ int msCGIDispatchBrowseRequest(mapservObj *mapserv) {
     return MS_FAILURE;
 
   /* -------------------------------------------------------------------- */
-  /*      generate map, legend, scalebar and refernce images.             */
+  /*      generate map, legend, scalebar and reference images.             */
   /* -------------------------------------------------------------------- */
   if (msGenerateImages(mapserv, MS_FALSE, MS_TRUE) != MS_SUCCESS)
     return MS_FAILURE;
@@ -2402,7 +2371,7 @@ int msCGIDispatchRequest(mapservObj *mapserv) {
 
 int msCGIHandler(const char *query_string, void **out_buffer,
                  size_t *buffer_length) {
-  int x, m = 0;
+  int m = 0;
   struct mstimeval execstarttime = {0}, execendtime = {0};
   struct mstimeval requeststarttime = {0}, requestendtime = {0};
   mapservObj *mapserv = NULL;
@@ -2445,7 +2414,7 @@ int msCGIHandler(const char *query_string, void **out_buffer,
 
   /* don't modify the string */
   queryString = msStrdup(query_string);
-  for (x = 0; queryString[0] != '\0'; x++) {
+  while (queryString[0] != '\0') {
     if (m >= maxParams) {
       maxParams *= 2;
       mapserv->request->ParamNames = (char **)realloc(
